public inbox for glibc-cvs@sourceware.org
help / color / mirror / Atom feed
From: Samuel Thibault <sthibaul@sourceware.org>
To: glibc-cvs@sourceware.org
Subject: [glibc] hurd: readv: Get rid of alloca
Date: Tue, 20 Jun 2023 17:15:20 +0000 (GMT) [thread overview]
Message-ID: <20230620171520.970F83858D1E@sourceware.org> (raw)
https://sourceware.org/git/gitweb.cgi?p=glibc.git;h=9e6863a537e66e01f5819dc356c5405a2bc67dc7
commit 9e6863a537e66e01f5819dc356c5405a2bc67dc7
Author: Joe Simmons-Talbott <josimmon@redhat.com>
Date: Mon Jun 19 10:43:34 2023 -0400
hurd: readv: Get rid of alloca
Replace alloca with a scratch_buffer to avoid potential stack overflows.
Checked on i686-gnu and x86_64-linux-gnu
Message-Id: <20230619144334.2902429-1-josimmon@redhat.com>
Diff:
---
sysdeps/posix/readv.c | 28 ++++++++++++----------------
1 file changed, 12 insertions(+), 16 deletions(-)
diff --git a/sysdeps/posix/readv.c b/sysdeps/posix/readv.c
index 91208e9894..9cdf571787 100644
--- a/sysdeps/posix/readv.c
+++ b/sysdeps/posix/readv.c
@@ -19,6 +19,7 @@
#include <unistd.h>
#include <string.h>
#include <limits.h>
+#include <scratch_buffer.h>
#include <stdbool.h>
#include <sys/param.h>
#include <sys/uio.h>
@@ -26,9 +27,9 @@
static void
-ifree (char **ptrp)
+ifree (struct scratch_buffer *sbuf)
{
- free (*ptrp);
+ scratch_buffer_free (sbuf);
}
/* Read data from file descriptor FD, and put the result in the
@@ -52,20 +53,15 @@ __readv (int fd, const struct iovec *vector, int count)
bytes += vector[i].iov_len;
}
- /* Allocate a temporary buffer to hold the data. We should normally
- use alloca since it's faster and does not require synchronization
- with other threads. But we cannot if the amount of memory
- required is too large. */
- char *buffer;
- char *malloced_buffer __attribute__ ((__cleanup__ (ifree))) = NULL;
- if (__libc_use_alloca (bytes))
- buffer = (char *) __alloca (bytes);
- else
- {
- malloced_buffer = buffer = (char *) malloc (bytes);
- if (buffer == NULL)
- return -1;
- }
+ /* Allocate a temporary buffer to hold the data. Use a scratch_buffer
+ since it's faster for small buffer sizes but can handle larger
+ allocations as well. */
+
+ struct scratch_buffer __attribute__ ((__cleanup__ (ifree))) buf;
+ scratch_buffer_init (&buf);
+ if (!scratch_buffer_set_array_size (&buf, 1, bytes))
+ return -1;
+ char *buffer = buf.data;
/* Read the data. */
ssize_t bytes_read = __read (fd, buffer, bytes);
reply other threads:[~2023-06-20 17:15 UTC|newest]
Thread overview: [no followups] expand[flat|nested] mbox.gz Atom feed
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20230620171520.970F83858D1E@sourceware.org \
--to=sthibaul@sourceware.org \
--cc=glibc-cvs@sourceware.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).